import numpy as np
import pylab as pl
import pickle as pk

ANTENNAS = ['gs','k2','mg','oe','ow']
ANTENNA = ANTENNAS[2]
EXPERIMENT = 'vo2187'

PLIM = 0.7



INFILE = 'ANTABS/%s%s.antabfs'%(EXPERIMENT,ANTENNA)


INP = open(INFILE,'r')
lines = INP.readlines()


# Get frequencies in right order:

IS_ONSALA = ANTENNA in ['oe','ow']
if not IS_ONSALA:

  IFS = {}
  for line in lines:
    if "!Column" in line and "LOfreq" in line:
        temp = line.split()
        IFS[int(temp[1])] = float(temp[6])

  NCHAN = len(IFS.keys())//2
  SORTIF = np.zeros(NCHAN)
  for i in range(NCHAN):
    SORTIF[i] = IFS[2*i+1]
  TOSORT = np.argsort(SORTIF)

else:

  NPERBAND = 8
  NBAND = 4
  IFS = {}
  for line in lines:
    if "!Column" in line and "LOfreq" in line:
       temp = line.split()
       IFS[int(temp[1])] = float(temp[6])

  NCHAN = NPERBAND*NBAND ##  len(IFS.keys())//2
  SORTIF = np.zeros(NCHAN)
  for j in range(NBAND):
    for i in range(NPERBAND):
      SORTIF[i+NPERBAND*j] = IFS[i+2*NPERBAND*j+1]
  TOSORT = np.argsort(SORTIF)




# Get scans:
START = []
for i,line in enumerate(lines):
    if "scanNum" in line and "scanName" in line and "source" in line:
        START.append(i)
START.append(len(lines))

#NCHAN = (len(lines[START[0]+1].split())-2)//2
NSCAN = len(START)-1

TSYS_X = np.zeros((NSCAN,NCHAN))
TSYS_Y = np.zeros((NSCAN,NCHAN))
AUX_X = np.zeros(NCHAN)
AUX_Y = np.zeros(NCHAN)
NGOOD = np.zeros(NCHAN,dtype=np.int32)


# Average Tsys over each scan:
ANTAB = {'SOURCE':[],'TIME':[],'TSYS_X':TSYS_X,'TSYS_Y':TSYS_Y,'FREQS':np.copy(SORTIF[TOSORT])}
for i in range(NSCAN):
    temp = (lines[START[i]].replace("\n","")).split()
    temp_i = (lines[START[i]+1].replace("\n","")).split()
    temp_f = (lines[START[i+1]-1].replace("\n","")).split()
    DAY = float(temp_i[0])
    DAY2 = float(temp_f[0])
    if i==0:
        DAY0 = DAY
    temp_i2 = list(map(float,temp_i[1].split(":")[:3]))
    temp_f2 = list(map(float,temp_f[1].split(":")[:3]))
    HOUR = temp_i2[0]+temp_i2[1]/60.+temp_i2[2]/3600. + 24.*(DAY-DAY0)
    HOUR2 = temp_f2[0]+temp_f2[1]/60.+temp_f2[2]/3600. + 24.*(DAY2-DAY0)
    ANTAB['SOURCE'].append(temp[-1].split("=")[1])
    ANTAB['TIME'].append([HOUR,HOUR2])
    AUX_X[:] = 0.0 ; AUX_Y[:] = 0.0 ; NGOOD[:] = 0
    for j in range(START[i]+1,START[i+1]):
        hdp = lines[j].split()[2:]
     #   print(hdp)
        temp = list(map(float,hdp))

        if IS_ONSALA:
          m = 0
          for k in range(NBAND):
            for l in range(NPERBAND):
               if temp[l + 2*NPERBAND*k]>0.0 and temp[l + NPERBAND*(2*k+1)]>0.0:
                 AUX_X[m] = temp[l + 2*NPERBAND*k]
                 AUX_Y[m] = temp[l + NPERBAND*(2*k+1)]
                 NGOOD[m] += 1
               m += 1
        else:
          for m in range(NCHAN):
            if temp[2*m]>0.0 and temp[2*m+1]>0.0:
              AUX_X[m] += temp[2*m]
              AUX_Y[m] += temp[2*m+1]
              NGOOD[m] += 1
    NGOOD[NGOOD==0] = 1
    TSYS_X[i,:] = AUX_X[TOSORT]/NGOOD[TOSORT]  #/(START[i+1]-START[i])
    TSYS_Y[i,:] = AUX_Y[TOSORT]/NGOOD[TOSORT]  #/(START[i+1]-START[i])
ANTAB['TSYS_X'] = TSYS_X
ANTAB['TSYS_Y'] = TSYS_Y
ANTAB['TIME'] = np.array(ANTAB['TIME'])

OFF = open("%s%s_ANTAB.dict"%(EXPERIMENT,ANTENNA),"wb")
pk.dump(ANTAB,OFF)
OFF.close()
        
RATIO = np.log10(ANTAB['TSYS_X']/ANTAB['TSYS_Y'])
fig = pl.figure(figsize=(10,5))
sub = fig.add_subplot(111)
fig.subplots_adjust(top=0.950,bottom=0.020,left=0.060,right=1.00)

pl.imshow(RATIO.transpose(),origin='lower',extent=(0.,1.,0.,1.),vmin=-PLIM,vmax=PLIM,aspect=0.5,interpolation='nearest',cmap='seismic')

fs = 18; fs2 = 25.
pl.xlabel('SCAN',fontsize=fs)
pl.ylabel('CHANNEL',fontsize=fs)
pl.setp(sub.get_xticklabels(),'visible',False)
pl.setp(sub.get_yticklabels(),'visible',False)
fig.suptitle("%s - %s  (X/Y Tsys)"%(ANTENNA,EXPERIMENT),fontsize=fs2)
cb = pl.colorbar()
cb.set_label('log10(X/Y)',fontsize=fs)
pl.savefig('%s_%s_TsysRatio.png'%(EXPERIMENT,ANTENNA))
pl.show()

        
    
    
