import numpy as np
import pylab as pl
import os, sys
import pickle as pk



SOURCES = ['1803+784','1849+670','2229+695','0059+581','0613+570','3C418','OJ287']
COLORS = ['r','g','r','k','orange','c','m']
#SOURCES = ['OJ287']
FLAGBAS = [['OE','OW']]

ANT_GAINS = {}
DEFAULT_GAIN = 1.0

EXPERIMENT = 'VO2187'


BAND = {'Cl':[5.e9,6.e9], 'Ch':[6.1e9,7.e9], 'X':[10.e9,11.e9], 'S':[3.e9,4.e9]}
SYMBOLS = ['x','+','o','^']
TPLOT = {}

ANTOBS = {}

for si in SOURCES:
    print("DOING source %s"%si)
    TPLOT[si] = {}

## Copy data:
    VIS = '%s_%s_AMPCAL.ms'%(EXPERIMENT,si)
    VIS0 = '%s_%s_AVER.ms'%(EXPERIMENT,si)
    os.system('rm -rf %s'%VIS)
    os.system('cp -r %s %s'%(VIS0,VIS))
     
    
## Figure out antenna names:
    tb.open(os.path.join(VIS,'ANTENNA'),nomodify=False)
    NAM = tb.getcol('NAME')
#    STA = tb.getcol('STATION')
#    NAM[:] = STA
#    tb.putcol('NAME',NAM)
    tb.close()

## Figure out IF frequencies:
    tb.open(os.path.join(VIS,'SPECTRAL_WINDOW'))
    NUS = tb.getcol('REF_FREQUENCY')
    tb.close()

# Read Tsys and set gains:
    TSYS = []
    TMED = []
    HAS_TSYS = []
    for i,ai in enumerate(NAM):
        ANTAB_NAME = '%s%s_ANTAB.dict'%(EXPERIMENT.lower(),ai.lower())
        if os.path.exists(ANTAB_NAME):
            IFF = open('%s%s_ANTAB.dict'%(EXPERIMENT.lower(),ai.lower()),'rb')
            TEMP = pk.load(IFF); IFF.close()
            TSYS.append(TEMP)
            TMED.append(np.median(np.sqrt(TEMP['TSYS_X']*TEMP['TSYS_X']),axis=0))
            HAS_TSYS.append(i)
            if ai not in ANTOBS.keys():
                ANTOBS[ai] = []
            for soui in TEMP['SOURCE']:
                if soui not in ANTOBS[ai]:
                    ANTOBS[ai].append(str(soui))
        else:
            TSYS.append({'FREQS':np.array([0.]), 'TIME':np.array([[0.,1.e20]]), 
                         'TSYS_X':np.array([[1.,1.]]),'TSYS_Y':np.array([[1.,1.]]),
                         'SOURCE':['NO_TSYS']})
            TMED.append(np.array([1.]))
        if ai not in ANT_GAINS.keys():
            ANT_GAINS[ai] = float(DEFAULT_GAIN)
    ANT_GAINS_ID = {}
    for ai in range(len(NAM)):
        ANT_GAINS_ID[ai] = ANT_GAINS[NAM[ai]]
    

## Plot Tsys:
    for bi in BAND.keys():
        TPLOT[si][bi] = {}
    silow = si.lower()
    for i in HAS_TSYS:
        TID = np.where(np.array(TSYS[i]['SOURCE'])==silow)[0]
        for bi in BAND.keys():
            MASK = np.logical_and(TSYS[i]['FREQS']*1.e6>BAND[bi][0],TSYS[i]['FREQS']*1.e6<BAND[bi][1])
            TPLOT[si][bi][NAM[i]] = [TSYS[i]['TIME'][TID], np.average(TSYS[i]['TSYS_X'][:,MASK],axis=1)[TID]]


  #  print("HDP 1")
        
## Figure out which IF corresponds to each channel:

    TNUS = [1.e6*TSYS[ni]['FREQS'] for ni in range(len(NAM))]
    IDX = np.zeros((len(NAM),len(NUS)),dtype=np.int32)
    for mi in range(len(NAM)):
        IDX[mi,:] = [np.argmin(np.abs(ni-TNUS[mi])) for ni in NUS]

## Get median Tsys for each IF:
    MED_IF = np.zeros(len(NUS))
    for i in range(len(NUS)):
        aux = []
        for ai in HAS_TSYS:
            aux.append(TMED[ai][IDX[ai,i]])
        MED_IF[i] = np.median(aux)

 #   print("Median Tsys per IF: ",MED_IF)


## Read data:
    tb.open(VIS,nomodify=False)
    SPI = tb.getcol('DATA_DESC_ID')
    A1 = tb.getcol('ANTENNA1')
    A2 = tb.getcol('ANTENNA2')
    TIME = tb.getcol('TIME')
    D0 = np.floor(np.min(TIME)/86400.)*86400.
    UT = (TIME-D0)/86400.*24.
    DATA = tb.getcol('DATA')
    Nvis = np.shape(DATA)[2]


  #  print("HDP 2")


## Flag autocorrs:
    FG = tb.getcol('FLAG')
    FG[:,:,A1==A2] = True

## Flag bad baselines:
    lNAM = list(NAM)
    for bas in FLAGBAS:
        if bas[0] in lNAM and bas[1] in lNAM:
            i1 = lNAM.index(bas[0])
            i2 = lNAM.index(bas[1])
            FG[:,:,np.logical_or((A1==i1)*(A2==i2),(A1==i2)*(A2==i1))] = True

    tb.putcol('FLAG',FG)

  #  print("HDP 3")

## Calibrate data:
    for i in range(Nvis):
        a1 = A1[i]; a2 = A2[i]; ts1 = TSYS[a1]; ts2 = TSYS[a2]
        sp = SPI[i]
   # for si,s in enumerate(ts1['TIME']):
   #     if UT[i]
    
   #     print("HDP 4 (%i): a1: %i  a2: %i | sp: %i"%(i,a1,a2,sp), a1 in HAS_TSYS, a2 in HAS_TSYS)

        if a1 in HAS_TSYS:
            idx1 = np.argmax(np.logical_and(UT[i]>=ts1['TIME'][:,0],UT[i]<=ts1['TIME'][:,1]))
            CORR1 = np.sqrt((ts1['TSYS_X'][idx1,IDX[a1,sp]])*(ts1['TSYS_X'][idx1,IDX[a1,sp]]))
        else:
            CORR1 = np.sqrt(MED_IF[sp])
            idx = 0; sp = 0

        if a2 in HAS_TSYS:
            idx2 = np.argmax(np.logical_and(UT[i]>=ts2['TIME'][:,0],UT[i]<=ts2['TIME'][:,1])) 
            CORR2 = np.sqrt((ts2['TSYS_X'][idx2,IDX[a2,sp]])*(ts2['TSYS_X'][idx2,IDX[a2,sp]]))
        else:
            CORR2 = np.sqrt(MED_IF[sp])
            idx = 0; sp = 0

      #  if i in [0,10,20]:
      #      print(ts1['SOURCE'][idx1], ts2['SOURCE'][idx2], UT[i], ts1['TIME'][idx1,:])  
#    idx1 = np.argmin(np.abs(ts1['TIME']-UT[i]))
#    idx2 = np.argmin(np.abs(ts2['TIME']-UT[i]))
        
        CORRFAC = np.sqrt(CORR1*CORR2/(ANT_GAINS_ID[a1]*ANT_GAINS_ID[a2]))
        DATA[:,:,i] *= CORRFAC
    
    tb.putcol('DATA',DATA)
    tb.close()
    
  #  print("HDP 5")
    

fig = pl.figure()
sub = fig.add_subplot(111)
TITLE = fig.suptitle('')
for anti in ANTOBS.keys():
    MAX = 0.0
    sub.cla()
    TITLE.set_text(anti.upper())
    for soui in ANTOBS[anti]:
        SOUI = soui.upper()
        if SOUI in SOURCES:
            SIDX = SOURCES.index(SOUI)
            sub.plot([],[],marker='o',c=COLORS[SIDX],label=SOUI,linestyle='none')
            for bii,bi in enumerate(sorted(BAND.keys())):
                sub.plot(TPLOT[SOUI][bi][anti][0][:,0],TPLOT[SOUI][bi][anti][1],marker=SYMBOLS[bii],c=COLORS[SIDX],linestyle='none')
                MAX = np.max([MAX,np.max(TPLOT[SOUI][bi][anti][1])])
    for bii,bi in enumerate(sorted(BAND.keys())):
        sub.plot([],[],'%sk'%(SYMBOLS[bii]),label=bi,linestyle='none')
    pl.legend(numpoints=1,ncol=4,loc=4)
    sub.set_xlabel('UT')
    sub.set_ylabel('Tsys (K)')
    sub.set_ylim((-30.,MAX*1.1))
    pl.savefig('ANTABS_ALLSOUR_%s.png'%anti)




os.system('rm -rf %s_AMPCAL.ms'%EXPERIMENT)
concat(vis = ['%s_%s_AMPCAL.ms'%(EXPERIMENT,si) for si in SOURCES],
       concatvis = '%s_AMPCAL.ms'%EXPERIMENT)










