import numpy as np
import pylab as pl
import os, sys
import pickle as pk


mysteps = [0,1,2,3,4]




SOURCES = ['1803+784','1849+670','2229+695','0059+581','0613+570','3C418','OJ287']
#SOURCES = ['1803+784']

EXPERIMENT = 'VO2187'


#SPW = '8~15'; SPNAME = 'Cl'; REFNU = '5GHz'
#CELL = '0.0001arcsec'
#AMPCAL = ['3600s','1800s','600s','300s']


#SPW = '24~31'; SPNAME = 'X'; REFNU = '10GHz'
#CELL = '0.000025arcsec'
#AMPCAL = ['3600s','1800s','600s']


SPW = '0~7'; SPNAME = 'S'; REFNU = '3GHz'
CELL = '0.0001arcsec'
AMPCAL = ['3600s','1800s','600s']



NTERMS = 1; NITER = 10000; GAIN = 0.02




INTERACT = True

NPHASECAL = 3

IMSIZE = 1024
ROBUST = 2.0






FLAGS = '%s_%s_FLAGS.dat'%(EXPERIMENT,SPNAME)
VIS = '%s_AMPCAL.ms'%EXPERIMENT


thesteps = {0: "Start up.",
           1: "Global scale (to 1 Jy).",
           2: "Phase self-cal.",
           3: "Amplitude self-cal.",
           4: "Final CLEAN."}


SPVIS = '%s_%s'%(SPNAME,VIS)
if 0 in mysteps:
    print(thesteps[0])
    os.system('rm -rf %s*'%SPVIS)
    split(vis=VIS,outputvis=SPVIS,spw=SPW,datacolumn='data')

## Apply flags (if there are):
    if os.path.exists(FLAGS):
        print('Will apply pre-existing flags')
        IFF = open(FLAGS,'rb')
        FGARR = pk.load(IFF)
        IFF.close()
        for k in [1,2,3]:
            FGARR[k,:,:] = FGARR[0,:,:]
        tb.open(SPVIS,nomodify=False)
        FGS = tb.getcol('FLAG')
        FGS[:] = FGARR
        tb.putcol('FLAG',FGS)
        tb.close()
      

tb.open(os.path.join(SPVIS,'SPECTRAL_WINDOW'))
NSPW = len(tb.getcol('REF_FREQUENCY'))
tb.close()




if 1 in mysteps:
    print(thesteps[1])
    clearcal(vis=SPVIS,addmodel=True)
    os.system("rm -rf GSCALE_%s.tab"%SPNAME)
    for hdp,si in enumerate(SOURCES):
        gaincal(vis=SPVIS, gaintype='T', solint = 'inf',
            calmode = 'a', field=si, 
            caltable = 'GSCALE_%s.tab'%SPNAME, append=hdp>0,
            combine = 'spw,scan')
        applycal(vis=SPVIS, gaintable = 'GSCALE_%s.tab'%SPNAME,
             spwmap = [0 for i in range(NSPW)],
             applymode='calonly', flagbackup=False,
             field=si,gainfield=si)
    flagmanager(vis=SPVIS, mode='save',versionname='STEP1')

ALLCALIB = ['GSCALE_%s.tab'%SPNAME]




if 2 in mysteps:
    print(thesteps[2])
    os.system('rm -rf PHASE_SELFCAL_*.tab')
    if 1 not in mysteps:
        flagmanager(vis=SPVIS, mode='restore',versionname='STEP1')
        for si in SOURCES:
            applycal(vis=SPVIS, gaintable = ALLCALIB,
                 field = si, gainfield = si,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 applymode='calonly', flagbackup=False)

    for hdp,si in enumerate(SOURCES):
        ALLCALIB = ['GSCALE_%s.tab'%SPNAME]
        for psc in range(NPHASECAL):
            imname = "%s_%s-%s"%(EXPERIMENT,si,SPNAME)
            os.system('rm -rf %s.*'%imname)
            if psc == 0:
                if not os.path.exists('%s_%s.msk'%(SPNAME,si)):
                    mask = ''
                else:
                    mask = '%s_%s.msk'%(SPNAME,si)
            else:
                mask = '%s_%s.msk'%(SPNAME,si)
            tclean(vis=SPVIS, field=si, datacolumn='corrected',
                 imagename=imname, imsize=IMSIZE, cell=CELL, 
                 specmode='mfs', nterms = NTERMS, stokes='I',
                 reffreq=REFNU, deconvolver = 'hogbom',
                 weighting='briggs', robust=ROBUST,
                 usemask='user', interactive=INTERACT,
                 mask = mask, niter=NITER, gain=GAIN,
                 savemodel='modelcolumn')
            os.system('rm -rf %s_%s.msk'%(SPNAME,si))
            os.system('cp -r %s.mask %s_%s.msk'%(imname,SPNAME,si))
            gaincal(vis=SPVIS, field=si, gaintable = ALLCALIB,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 gainfield=si, minblperant=3,
                 combine = 'spw', gaintype='T', calmode = 'p', append = hdp>0,
                 caltable = 'PHASE_SELFCAL_%s_%i.tab'%(SPNAME,psc),solint='inf')
            ALLCALIB.append('PHASE_SELFCAL_%s_%i.tab'%(SPNAME,psc))
            applycal(vis=SPVIS, gaintable = ALLCALIB, field=si, gainfield=si,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 interp = 'nearest',
                 applymode='calonly', flagbackup=False)
    flagmanager(vis=SPVIS, mode='save',versionname='STEP2')


ALLCALIB = ['GSCALE_%s.tab'%SPNAME] + ['PHASE_SELFCAL_%s_%i.tab'%(SPNAME,i) for i in range(NPHASECAL)]





if 3 in mysteps:
    print(thesteps[3])
    os.system('rm -rf AMP_SELFCAL_*.tab')
    if 2 not in mysteps:
        flagmanager(vis=SPVIS, mode='restore',versionname='STEP2')
        for si in SOURCES:
            applycal(vis=SPVIS, gaintable = ALLCALIB,
                 field = si, gainfield = si, interp='nearest',
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 applymode='calonly', flagbackup=False)

    for hdp,si in enumerate(SOURCES):
        ALLCALIB = ['GSCALE_%s.tab'%SPNAME] + ['PHASE_SELFCAL_%s_%i.tab'%(SPNAME,i) for i in range(NPHASECAL)]
        for psc in range(len(AMPCAL)):
            imname = "%s_%s-%s"%(EXPERIMENT,si,SPNAME)
            os.system('rm -rf %s.*'%imname)
            if psc == 0:
                if not os.path.exists('%s_%s.msk'%(SPNAME,si)):
                    mask = ''
                else:
                    mask = '%s_%s.msk'%(SPNAME,si)
            else:
                mask = '%s_%s.msk'%(SPNAME,si)
            tclean(vis=SPVIS, field=si, datacolumn='corrected',
                 imagename=imname, imsize=IMSIZE, cell=CELL, 
                 specmode='mfs', nterms = NTERMS, stokes='I',
                 reffreq=REFNU, deconvolver = 'hogbom',
                 weighting='briggs', robust=ROBUST,
                 usemask='user', interactive=INTERACT,
                 mask = mask, niter=NITER, gain=GAIN,
                 savemodel='modelcolumn')
            os.system('rm -rf %s_%s.msk'%(SPNAME,si))
            os.system('cp -r %s.mask %s_%s.msk'%(imname,SPNAME,si))
            gaincal(vis=SPVIS, field=si, gaintable = ALLCALIB,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 gainfield=si, minblperant=3,
                 interp = ['nearest' for kk in range(1+NPHASECAL)] + ['linear' for kk in range(psc)],
                 combine = 'spw,scan', gaintype='T', calmode = 'ap', append = hdp>0,
                 caltable = 'AMP_SELFCAL_%s_%i.tab'%(SPNAME,psc),solint=AMPCAL[psc])
            ALLCALIB.append('AMP_SELFCAL_%s_%i.tab'%(SPNAME,psc))
            applycal(vis=SPVIS, gaintable = ALLCALIB, field=si, gainfield=si,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 interp = ['nearest' for kk in range(1+NPHASECAL)] + ['linear' for kk in range(psc+1)],                 
                 applymode='calonly', flagbackup=False)


ALLCALIB = ['GSCALE_%s.tab'%SPNAME] + ['PHASE_SELFCAL_%s_%i.tab'%(SPNAME,i) for i in range(NPHASECAL)]
ALLCALIB += ['AMP_SELFCAL_%s_%i.tab'%(SPNAME,i) for i in range(len(AMPCAL))]
INTERP = ['nearest' for kk in range(1+NPHASECAL)] + ['linear' for kk in range(len(AMPCAL))]


if 4 in mysteps:
    print(thesteps[4])
    if 3 not in mysteps:
        flagmanager(vis=SPVIS, mode='restore',versionname='STEP2')
        for si in SOURCES:
            applycal(vis=SPVIS, gaintable = ALLCALIB,
                 field = si, gainfield = si, interp=INTERP,
                 spwmap = [[0 for i in range(NSPW)] for kk in ALLCALIB],
                 applymode='calonly', flagbackup=False)
    for hdp,si in enumerate(SOURCES):
            imname = "%s_%s-%s"%(EXPERIMENT,si,SPNAME)
            os.system('rm -rf %s.*'%imname)
            mask = '%s_%s.msk'%(SPNAME,si)
            tclean(vis=SPVIS, field=si, datacolumn='corrected',
                 imagename=imname, imsize=IMSIZE, cell=CELL, 
                 specmode='mfs', nterms = NTERMS, stokes='I',
                 reffreq=REFNU, deconvolver = 'hogbom',
                 weighting='briggs', robust=ROBUST,
                 usemask='user', interactive=False,
                 mask = mask, niter=NITER, gain=GAIN,
                 savemodel='modelcolumn')



## TODO:  RUN POLSOLVE.         




